@@ -13,10 +13,9 @@
 # limitations under the License.
 
 import ibis
+from data_validation import clients, consts
 from ibis.expr.types import StringScalar
-from third_party.ibis.ibis_addon import operations
-
-from data_validation import clients
+from third_party.ibis.ibis_addon import api, operations
 
 
 class AggregateField(object):
@@ -102,12 +101,12 @@ class AggregateField(object):
         else:
             agg_field = self.expr(ibis_table)
 
-        if self.alias:
-            agg_field = agg_field.name(self.alias)
-
         if self.cast:
             agg_field = agg_field.cast(self.cast)
 
+        if self.alias:
+            agg_field = agg_field.name(self.alias)
+
         return agg_field
 
 
@@ -154,6 +153,13 @@ class FilterField(object):
             ibis.expr.types.ColumnExpr.__eq__, left_field=field_name, right=value
         )
 
+    @staticmethod
+    def isin(field_name, values):
+        # Build Left and Right Objects
+        return FilterField(
+            ibis.expr.types.ColumnExpr.isin, left_field=field_name, right=values
+        )
+
     @staticmethod
     def custom(expr):
         """Returns a FilterField instance built for any custom SQL using a supported operator.
@@ -185,6 +191,28 @@ class FilterField(object):
         return self.expr(self.left, self.right)
 
 
+class ComparisonField(object):
+    def __init__(self, field_name, alias=None, cast=None):
+        """A representation of a comparison field used to build a query.
+
+        Args:
+            field_name (String): A field to act on in the table
+            alias (String): An alias to use for the group
+            cast (String): A cast on the column if required
+        """
+        self.field_name = field_name
+        self.alias = alias
+        self.cast = cast
+
+    def compile(self, ibis_table):
+        # Fields are supplied on compile or on build
+        comparison_field = ibis_table[self.field_name]
+        alias = self.alias or self.field_name
+        comparison_field = comparison_field.name(alias)
+
+        return comparison_field
+
+
 class GroupedField(object):
     def __init__(self, field_name, alias=None, cast=None):
         """A representation of a group by field used to build a query.
@@ -221,7 +249,7 @@ class GroupedField(object):
 
 class ColumnReference(object):
     def __init__(self, column_name):
-        """ A representation of an calculated field to build a query.
+        """A representation of an calculated field to build a query.
 
         Args:
             column_name (String): The column name used in a complex expr
@@ -229,7 +257,7 @@ class ColumnReference(object):
         self.column_name = column_name
 
     def compile(self, ibis_table):
-        """ Return an ibis object referencing the column.
+        """Return an ibis object referencing the column.
 
         Args:
             ibis_table (IbisTable): The table obj reference
@@ -240,7 +268,7 @@ class ColumnReference(object):
 class CalculatedField(object):
     def __init__(self, ibis_expr, config, fields, cast=None, **kwargs):
 
-        """ A representation of an calculated field to build a query.
+        """A representation of an calculated field to build a query.
 
         Args:
             config dict: Configurations object explaining calc field details
@@ -255,41 +283,94 @@ class CalculatedField(object):
     @staticmethod
     def concat(config, fields):
         if config.get("default_concat_separator") is None:
-            config["default_concat_separator"] = ibis.literal(",")
+            config["default_concat_separator"] = ibis.literal("")
         fields = [config["default_concat_separator"], fields]
         cast = "string"
         return CalculatedField(
-            ibis.expr.api.StringValue.join, config, fields, cast=cast,
+            ibis.expr.api.StringValue.join,
+            config,
+            fields,
+            cast=cast,
         )
 
     @staticmethod
     def hash(config, fields):
         if config.get("default_hash_function") is None:
+            how = "sha256"
+            return CalculatedField(
+                ibis.expr.api.StringValue.hashbytes,
+                config,
+                fields,
+                how=how,
+            )
+        else:
             how = "farm_fingerprint"
-        return CalculatedField(ibis.expr.api.ValueExpr.hash, config, fields, how=how,)
+            return CalculatedField(
+                ibis.expr.api.ValueExpr.hash,
+                config,
+                fields,
+                how=how,
+            )
 
     @staticmethod
     def ifnull(config, fields):
-        if config.get("default_null_string") is None:
-            config["default_string"] = ibis.literal("DEFAULT_REPLACEMENT_STRING")
-        fields = [config["default_string"], fields[0]]
-        return CalculatedField(ibis.expr.api.ValueExpr.fillna, config, fields,)
+        config["default_string"] = (
+            ibis.literal("DEFAULT_REPLACEMENT_STRING")
+            if config.get("default_null_string") is None
+            else config.get("default_null_string")
+        )
+        fields = [fields[0], config["default_string"]]
+        return CalculatedField(
+            ibis.expr.api.ValueExpr.fillna,
+            config,
+            fields,
+        )
 
     @staticmethod
     def length(config, fields):
-        return CalculatedField(ibis.expr.api.StringValue.length, config, fields,)
+        return CalculatedField(
+            ibis.expr.api.StringValue.length,
+            config,
+            fields,
+        )
 
     @staticmethod
     def rstrip(config, fields):
-        return CalculatedField(ibis.expr.api.StringValue.rstrip, config, fields,)
+        return CalculatedField(
+            ibis.expr.api.StringValue.rstrip,
+            config,
+            fields,
+        )
 
     @staticmethod
     def upper(config, fields):
-        return CalculatedField(ibis.expr.api.StringValue.upper, config, fields,)
+        return CalculatedField(
+            ibis.expr.api.StringValue.upper,
+            config,
+            fields,
+        )
+
+    @staticmethod
+    def epoch_seconds(config, fields):
+        return CalculatedField(
+            ibis.expr.api.TimestampValue.epoch_seconds,
+            config,
+            fields,
+        )
+
+    @staticmethod
+    def cast(config, fields):
+        target_type = config.get(consts.CONFIG_DEFAULT_CAST, "string")
+        return CalculatedField(
+            api.cast,
+            config,
+            fields,
+            target_type=target_type,
+        )
 
     @staticmethod
     def custom(expr):
-        """ Returns a CalculatedField instance built for any custom SQL using a supported operator.
+        """Returns a CalculatedField instance built for any custom SQL using a supported operator.
         Args:
             expr (Str): A custom SQL expression used to filter a query
         """
@@ -308,7 +389,6 @@ class CalculatedField(object):
                     compiled_fields.append(ibis_table[field].cast(self.cast))
                 else:
                     compiled_fields.append(ibis_table[field])
-
         return compiled_fields
 
     def compile(self, ibis_table):
@@ -322,9 +402,15 @@ class CalculatedField(object):
 
 class QueryBuilder(object):
     def __init__(
-        self, aggregate_fields, calculated_fields, filters, grouped_fields, limit=None
+        self,
+        aggregate_fields,
+        calculated_fields,
+        filters,
+        grouped_fields,
+        comparison_fields,
+        limit=None,
     ):
-        """ Build a QueryBuilder object which can be used to build queries easily
+        """Build a QueryBuilder object which can be used to build queries easily
 
         Args:
             aggregate_fields (list[AggregateField]): AggregateField instances with Ibis expressions
@@ -337,20 +423,23 @@ class QueryBuilder(object):
         self.calculated_fields = calculated_fields
         self.filters = filters
         self.grouped_fields = grouped_fields
+        self.comparison_fields = comparison_fields
         self.limit = limit
 
     @staticmethod
     def build_count_validator(limit=None):
-        """ Return a basic template builder for most validations """
+        """Return a basic template builder for most validations"""
         aggregate_fields = []
         filters = []
         grouped_fields = []
+        comparison_fields = []
         calculated_fields = []
 
         return QueryBuilder(
             aggregate_fields,
             filters=filters,
             grouped_fields=grouped_fields,
+            comparison_fields=comparison_fields,
             calculated_fields=calculated_fields,
         )
 
@@ -365,15 +454,23 @@ class QueryBuilder(object):
     def compile_group_fields(self, table):
         return [field.compile(table) for field in self.grouped_fields]
 
-    def compile_calculated_fields(self, table, n=None):
-        if n is not None:
-            return [
-                field.compile(table)
-                for field in self.calculated_fields
-                if field.config["depth"] == n
-            ]
-        else:
-            return [field.compile(table) for field in self.calculated_fields]
+    def compile_comparison_fields(self, table):
+        return [field.compile(table) for field in self.comparison_fields]
+
+    def compile_calculated_fields(self, table, n=0):
+        return [
+            field.compile(table)
+            for field in self.calculated_fields
+            if field.config[consts.CONFIG_DEPTH] == n
+        ]
+        # if n is not None:
+        #     return [
+        #         field.compile(table)
+        #         for field in self.calculated_fields
+        #         if field.config[consts.CONFIG_DEPTH] == n
+        #     ]
+        # else:
+        #     return [field.compile(table) for field in self.calculated_fields]
 
     def compile(self, data_client, schema_name, table_name):
         """Return an Ibis query object
@@ -389,7 +486,8 @@ class QueryBuilder(object):
         calc_table = table
         if self.calculated_fields:
             depth_limit = max(
-                field.config.get("depth", 0) for field in self.calculated_fields
+                field.config.get(consts.CONFIG_DEPTH, 0)
+                for field in self.calculated_fields
             )
             for n in range(0, (depth_limit + 1)):
                 calc_table = calc_table.mutate(
@@ -399,15 +497,18 @@ class QueryBuilder(object):
         filtered_table = (
             calc_table.filter(compiled_filters) if compiled_filters else calc_table
         )
-
         compiled_groups = self.compile_group_fields(filtered_table)
         grouped_table = (
             filtered_table.groupby(compiled_groups)
             if compiled_groups
             else filtered_table
         )
-
-        query = grouped_table.aggregate(self.compile_aggregate_fields(filtered_table))
+        if self.aggregate_fields:
+            query = grouped_table.aggregate(
+                self.compile_aggregate_fields(filtered_table)
+            )
+        else:
+            query = grouped_table
 
         if self.limit:
             query = query.limit(self.limit)
@@ -423,11 +524,19 @@ class QueryBuilder(object):
         """
         self.aggregate_fields.append(aggregate_field)
 
+    def add_comparison_field(self, comparison_field):
+        """Add an ComparisonField instance to the query which
+            will be used when compiling your query (ie. SUM(a))
+
+        Args:
+            comparison_field (ComparisonField): An ComparisonField instance
+        """
+        self.comparison_fields.append(comparison_field)
+
     def add_grouped_field(self, grouped_field):
         """Add a GroupedField instance to the query which
             represents adding a column to group by in the
             query being built.
-
         Args:
             grouped_field (GroupedField): A GroupedField instance
         """
@@ -437,14 +546,13 @@ class QueryBuilder(object):
         """Add a FilterField instance to your query which
             will add the desired filter to your compiled
             query (ie. WHERE query_filter=True)
-
         Args:
             filter_obj (FilterField): A FilterField instance
         """
         self.filters.append(filter_obj)
 
     def add_calculated_field(self, calculated_field):
-        """ Add a CalculatedField instance to your query which
+        """Add a CalculatedField instance to your query which
             will add the desired scalar function to your compiled
             query (ie. CONCAT(field_a, field_b))
         Args:
